<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Block\Multishipping\Checkout;

use Aheadworks\Afptc\Api\Data\RuleInterface;
use Aheadworks\Afptc\Api\RuleRepositoryInterface;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items\ScenarioPool;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\View\Element\Template;
use Magento\Framework\View\Element\Template\Context;
use Magento\Checkout\Model\Session as CheckoutSession;
use Magento\Quote\Model\Quote\Item\AbstractItem;

/**
 * Class Addresses
 *
 * @package Aheadworks\Afptc\Block\Multishipping\Checkout
 */
class Addresses extends Template
{
    /**
     * @var CheckoutSession
     */
    private $checkoutSession;

    /**
     * @var RuleRepositoryInterface
     */
    private $ruleRepository;

    /**
     * @var ScenarioPool
     */
    private $scenarioPool;

    /**
     * @param Context $context
     * @param CheckoutSession $checkoutSession
     * @param RuleRepositoryInterface $ruleRepository
     * @param ScenarioPool $scenarioPool
     * @param array $data
     */
    public function __construct(
        Context $context,
        CheckoutSession $checkoutSession,
        RuleRepositoryInterface $ruleRepository,
        ScenarioPool $scenarioPool,
        array $data = []
    ) {
        parent::__construct($context, $data);
        $this->checkoutSession = $checkoutSession;
        $this->ruleRepository = $ruleRepository;
        $this->scenarioPool = $scenarioPool;
    }

    /**
     * Retrieve pairs of promo and main products
     *
     * @return array
     */
    public function getDependencies()
    {
        $dependencies = [];
        $items = $this->checkoutSession->getQuote()->getAllItems();
        foreach ($items as $item) {
            if ($item->getAwAfptcIsPromo() && (float)$item->getAwAfptcQty()) {
                $rule = $this->getCurrentRule($item);
                try {
                    $processor = $this->scenarioPool->getScenarioProcessor($rule->getScenario());
                } catch (LocalizedException $e) {
                    continue;
                }

                $mainItems = $processor->getRuleItems($rule, $items);
                if (!$mainItems) {
                    continue;
                }

                $dependencies[] = [
                    'promo' => $item->getName(),
                    'main' => $this->convertToString($mainItems)
                ];
            }
        }

        return $dependencies;
    }

    /**
     * Get current promo rule
     *
     * @param AbstractItem $quoteItem
     * @return RuleInterface|null
     */
    private function getCurrentRule($quoteItem)
    {
        try {
            $appliedRuleIds = explode(",", $quoteItem->getAwAfptcRuleIds());
            $firstAppliedRule = reset($appliedRuleIds);
            $rule = $this->ruleRepository->get($firstAppliedRule);
        } catch (NoSuchEntityException $exception) {
            $rule = null;
        }

        return $rule;
    }

    /**
     * Retrieve main products names
     *
     * @param AbstractItem[] $items
     * @return string
     */
    public function convertToString($items)
    {
        $arrayOfNames = [];
        foreach ($items as $item) {
            $arrayOfNames[] = $item->getName();
        }

        return implode(', ', $arrayOfNames);
    }
}
