<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Cart\Converter\ToCartItem;

use Aheadworks\Afptc\Api\Data\RuleMetadataInterface;
use Aheadworks\Afptc\Api\Data\RuleMetadataPromoProductInterface;

/**
 * Class Validator
 * @package Aheadworks\Afptc\Model\Cart\Converter\ToCartItem
 */
class Validator
{
    /**
     * Check if valid metadata rule for add promo products to cart
     *
     * @param RuleMetadataInterface $metadataRule
     * @return bool
     */
    public function isValid($metadataRule)
    {
        $calculatedPromoProductsQty = $this->calculatePromoProductsQty($metadataRule->getPromoProducts());
        return $calculatedPromoProductsQty > 0 && $metadataRule->getAvailableQtyToGive() >= $calculatedPromoProductsQty;
    }

    /**
     * Calculate promo products qty
     *
     * @param RuleMetadataPromoProductInterface[] $promoProducts
     * @return float
     */
    private function calculatePromoProductsQty($promoProducts)
    {
        $qty = 0;
        foreach ($promoProducts as $promoProduct) {
            $qty += $promoProduct->getQty();
        }
        return $qty;
    }
}
