<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Cart\Item\ExtensionAttributes\Processor;

/**
 * Class Composite
 *
 * @package Aheadworks\Afptc\Model\Cart\Item\ExtensionAttributes\Processor
 */
class Composite implements ProcessorInterface
{
    /**
     * @var array[]
     */
    private $processors;

    /**
     * @param array $processors
     */
    public function __construct(array $processors = [])
    {
        $this->processors = $processors;
    }

    /**
     * {@inheritdoc}
     */
    public function prepareDataBeforeSave($item)
    {
        foreach ($this->processors as $processor) {
            $processor->prepareDataBeforeSave($item);
        }
        return $item;
    }

    /**
     * {@inheritdoc}
     */
    public function prepareDataAfterLoad($item)
    {
        foreach ($this->processors as $processor) {
            $processor->prepareDataAfterLoad($item);
        }
        return $item;
    }
}
