<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\ExtensionAttributes\Builder;

use Magento\Quote\Api\Data\CartExtensionFactory;
use Magento\Quote\Api\Data\CartItemExtensionFactory;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Quote\Api\Data\CartItemInterface;

/**
 * Class Cart
 * @package Aheadworks\Afptc\Model\ExtensionAttributes\Builder
 */
class Cart
{
    /**
     * @var CartExtensionFactory
     */
    private $cartExtensionFactory;

    /**
     * @var CartItemExtensionFactory
     */
    private $cartItemExtensionFactory;

    /**
     * @param CartExtensionFactory $cartExtensionFactory
     * @param CartItemExtensionFactory $cartItemExtensionFactory
     */
    public function __construct(
        CartExtensionFactory $cartExtensionFactory,
        CartItemExtensionFactory $cartItemExtensionFactory
    ) {
        $this->cartExtensionFactory = $cartExtensionFactory;
        $this->cartItemExtensionFactory = $cartItemExtensionFactory;
    }

    /**
     * Attach AW afptc attributes
     *
     * @param CartInterface $cart
     * @return CartInterface
     */
    public function attachAwAfptcAttributes($cart)
    {
        /** @var \Magento\Quote\Api\Data\CartExtension $cartExtension */
        $cartExtension = $cart->getExtensionAttributes()
            ? $cart->getExtensionAttributes()
            : $this->cartExtensionFactory->create();

        $cartExtension
            ->setAwAfptcAmount($cart->getAwAfptcAmount())
            ->setBaseAwAfptcAmount($cart->getBaseAwAfptcAmount())
            ->setAwAfptcUsesCoupon($cart->getAwAfptcUsesCoupon());

        $cart->setExtensionAttributes($cartExtension);

        $cartItems = $cart->getItems() ? : [];
        foreach ($cartItems as $item) {
            $this->attachAwAfptcAttributesToItem($item);
        }

        return $cart;
    }

    /**
     * Attach AW afptc attributes to item
     *
     * @param CartItemInterface $item
     * @return CartItemInterface
     */
    public function attachAwAfptcAttributesToItem($item)
    {
        /** @var \Magento\Quote\Api\Data\CartItemExtension $cartItemExtension */
        $cartItemExtension = $item->getExtensionAttributes()
            ? $item->getExtensionAttributes()
            : $this->cartItemExtensionFactory->create();

        $cartItemExtension
            ->setAwAfptcRuleIds($item->getAwAfptcRuleIds())
            ->setAwAfptcQty($item->getAwAfptcQty())
            ->setAwAfptcPercent($item->getAwAfptcPercent())
            ->setAwAfptcAmount($item->getAwAfptcAmount())
            ->setBaseAwAfptcAmount($item->getBaseAwAfptcAmount());

        $item->setExtensionAttributes($cartItemExtension);

        return $item;
    }
}
