<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\ExtensionAttributes\Builder;

use Magento\Sales\Api\Data\CreditmemoExtensionFactory;
use Magento\Sales\Api\Data\CreditmemoItemExtensionFactory;
use Magento\Sales\Api\Data\CreditmemoInterface;
use Magento\Sales\Api\Data\CreditmemoItemInterface;

/**
 * Class Creditmemo
 * @package Aheadworks\Afptc\Model\ExtensionAttributes\Builder
 */
class Creditmemo
{
    /**
     * @var CreditmemoExtensionFactory
     */
    private $creditmemoExtensionFactory;

    /**
     * @var CreditmemoItemExtensionFactory
     */
    private $creditmemoItemExtensionFactory;

    /**
     * @param CreditmemoExtensionFactory $creditmemoExtensionFactory
     * @param CreditmemoItemExtensionFactory $creditmemoItemExtensionFactory
     */
    public function __construct(
        CreditmemoExtensionFactory $creditmemoExtensionFactory,
        CreditmemoItemExtensionFactory $creditmemoItemExtensionFactory
    ) {
        $this->creditmemoExtensionFactory = $creditmemoExtensionFactory;
        $this->creditmemoItemExtensionFactory = $creditmemoItemExtensionFactory;
    }

    /**
     * Attach AW afptc attributes
     *
     * @param CreditmemoInterface $creditmemo
     * @return CreditmemoInterface
     */
    public function attachAwAfptcAttributes($creditmemo)
    {
        /** @var \Magento\Sales\Api\Data\CreditmemoExtension $creditmemoExtension */
        $creditmemoExtension = $creditmemo->getExtensionAttributes()
            ? $creditmemo->getExtensionAttributes()
            : $this->creditmemoExtensionFactory->create();

        $creditmemoExtension
            ->setAwAfptcAmount($creditmemo->getAwAfptcAmount())
            ->setBaseAwAfptcAmount($creditmemo->getBaseAwAfptcAmount());

        $creditmemo->setExtensionAttributes($creditmemoExtension);

        $items = $creditmemo->getItems() ? $creditmemo->getItems() : [];
        foreach ($items as $item) {
            $this->attachAwAfptcAttributesToItem($item);
        }

        return $creditmemo;
    }

    /**
     * Attach AW afptc attributes to item
     *
     * @param CreditmemoItemInterface $item
     * @return CreditmemoItemInterface
     */
    public function attachAwAfptcAttributesToItem($item)
    {
        /** @var \Magento\Sales\Api\Data\CreditmemoItemExtension $creditmemoItemExtension */
        $creditmemoItemExtension = $item->getExtensionAttributes()
            ? $item->getExtensionAttributes()
            : $this->creditmemoItemExtensionFactory->create();

        $creditmemoItemExtension
            ->setAwAfptcAmount($item->getAwAfptcAmount())
            ->setBaseAwAfptcAmount($item->getBaseAwAfptcAmount());

        $item->setExtensionAttributes($creditmemoItemExtension);

        return $item;
    }
}
