<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\ExtensionAttributes\Builder;

use Magento\Sales\Api\Data\InvoiceExtensionFactory;
use Magento\Sales\Api\Data\InvoiceItemExtensionFactory;
use Magento\Sales\Api\Data\InvoiceInterface;
use Magento\Sales\Api\Data\InvoiceItemInterface;

/**
 * Class Invoice
 * @package Aheadworks\Afptc\Model\ExtensionAttributes\Builder
 */
class Invoice
{
    /**
     * @var InvoiceExtensionFactory
     */
    private $invoiceExtensionFactory;

    /**
     * @var InvoiceItemExtensionFactory
     */
    private $invoiceItemExtensionFactory;

    /**
     * @param InvoiceExtensionFactory $invoiceExtensionFactory
     * @param InvoiceItemExtensionFactory $invoiceItemExtensionFactory
     */
    public function __construct(
        InvoiceExtensionFactory $invoiceExtensionFactory,
        InvoiceItemExtensionFactory $invoiceItemExtensionFactory
    ) {
        $this->invoiceExtensionFactory = $invoiceExtensionFactory;
        $this->invoiceItemExtensionFactory = $invoiceItemExtensionFactory;
    }

    /**
     * Attach AW afptc attributes
     *
     * @param InvoiceInterface $invoice
     * @return InvoiceInterface
     */
    public function attachAwAfptcAttributes($invoice)
    {
        /** @var \Magento\Sales\Api\Data\InvoiceExtension $invoiceExtension */
        $invoiceExtension = $invoice->getExtensionAttributes()
            ? $invoice->getExtensionAttributes()
            : $this->invoiceExtensionFactory->create();

        $invoiceExtension
            ->setAwAfptcAmount($invoice->getAwAfptcAmount())
            ->setBaseAwAfptcAmount($invoice->getBaseAwAfptcAmount());

        $invoice->setExtensionAttributes($invoiceExtension);

        $items = $invoice->getItems() ? $invoice->getItems() : [];
        foreach ($items as $item) {
            $this->attachAwAfptcAttributesToItem($item);
        }

        return $invoice;
    }

    /**
     * Attach AW afptc attributes to item
     *
     * @param InvoiceItemInterface $item
     * @return InvoiceItemInterface
     */
    public function attachAwAfptcAttributesToItem($item)
    {
        /** @var \Magento\Sales\Api\Data\InvoiceItemExtension $invoiceItemExtension */
        $invoiceItemExtension = $item->getExtensionAttributes()
            ? $item->getExtensionAttributes()
            : $this->invoiceItemExtensionFactory->create();

        $invoiceItemExtension
            ->setAwAfptcAmount($item->getAwAfptcAmount())
            ->setBaseAwAfptcAmount($item->getBaseAwAfptcAmount());

        $item->setExtensionAttributes($invoiceItemExtension);

        return $item;
    }
}
