<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\ExtensionAttributes\Builder;

use Magento\Sales\Api\Data\OrderExtensionFactory;
use Magento\Sales\Api\Data\OrderItemExtensionFactory;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\Data\OrderItemInterface;

/**
 * Class Order
 * @package Aheadworks\Afptc\Model\ExtensionAttributes\Builder
 */
class Order
{
    /**
     * @var OrderExtensionFactory
     */
    private $orderExtensionFactory;

    /**
     * @var OrderItemExtensionFactory
     */
    private $orderItemExtensionFactory;

    /**
     * @param OrderExtensionFactory $orderExtensionFactory
     * @param OrderItemExtensionFactory $orderItemExtensionFactory
     */
    public function __construct(
        OrderExtensionFactory $orderExtensionFactory,
        OrderItemExtensionFactory $orderItemExtensionFactory
    ) {
        $this->orderExtensionFactory = $orderExtensionFactory;
        $this->orderItemExtensionFactory = $orderItemExtensionFactory;
    }

    /**
     * Attach AW afptc attributes
     *
     * @param OrderInterface $order
     * @return OrderInterface
     */
    public function attachAwAfptcAttributes($order)
    {
        /** @var \Magento\Sales\Api\Data\OrderExtension $orderExtension */
        $orderExtension = $order->getExtensionAttributes()
            ? $order->getExtensionAttributes()
            : $this->orderExtensionFactory->create();

        $orderExtension
            ->setAwAfptcAmount($order->getAwAfptcAmount())
            ->setBaseAwAfptcAmount($order->getBaseAwAfptcAmount())
            ->setAwAfptcInvoiced($order->getAwAfptcInvoiced())
            ->setBaseAwAfptcInvoiced($order->getBaseAwAfptcInvoiced())
            ->setAwAfptcRefunded($order->getAwAfptcRefunded())
            ->setBaseAwAfptcRefunded($order->getBaseAwAfptcRefunded());

        $order->setExtensionAttributes($orderExtension);

        $items = $order->getItems() ? $order->getItems() : [];
        foreach ($items as $item) {
            $this->attachAwAfptcAttributesToItem($item);
        }

        return $order;
    }

    /**
     * Attach AW afptc attributes to item
     *
     * @param OrderItemInterface $item
     * @return OrderItemInterface
     */
    public function attachAwAfptcAttributesToItem($item)
    {
        /** @var \Magento\Sales\Api\Data\OrderItemExtension $orderItemExtension */
        $orderItemExtension = $item->getExtensionAttributes()
            ? $item->getExtensionAttributes()
            : $this->orderItemExtensionFactory->create();

        $orderItemExtension
            ->setAwAfptcRuleIds($item->getAwAfptcRuleIds())
            ->setAwAfptcQty($item->getAwAfptcQty())
            ->setAwAfptcQtyInvoiced($item->getAwAfptcQtyInvoiced())
            ->setAwAfptcQtyRefunded($item->getAwAfptcQtyRefunded())
            ->setAwAfptcIsPromo($item->getAwAfptcIsPromo())
            ->setAwAfptcPercent($item->getAwAfptcPercent())
            ->setAwAfptcAmount($item->getAwAfptcAmount())
            ->setBaseAwAfptcAmount($item->getBaseAwAfptcAmount())
            ->setAwAfptcInvoiced($item->getAwAfptcInvoiced())
            ->setBaseAwAfptcInvoiced($item->getBaseAwAfptcInvoiced())
            ->setAwAfptcRefunded($item->getAwAfptcRefunded())
            ->setBaseAwAfptcRefunded($item->getBaseAwAfptcRefunded());

        $item->setExtensionAttributes($orderItemExtension);

        return $item;
    }
}
