<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\ResourceModel\Product;

use Magento\Catalog\Model\ResourceModel\Product\Collection as ProductCollection;
use Magento\Downloadable\Model\Product\Type as DownloadableProduct;
use Magento\Framework\Exception\LocalizedException;

/**
 * Class Collection
 * @package Aheadworks\Afptc\Model\ResourceModel\Product
 */
class Collection extends ProductCollection
{
    /**
     * Exclude products with required options
     *
     * @return $this
     */
    public function excludeProductsWithRequiredOptions()
    {
        try {
            $mainTableAlias = self::MAIN_TABLE_ALIAS;
            $linkField = $this->getEntity()->getLinkField();

            $conditions = [];
            $conditions[] = "`$mainTableAlias`.`required_options` != 1";
            $conditions[] = "`$mainTableAlias`.`required_options` IS NULL";
            $conditions[] = "`cpo`.`option_id` IS NULL";
            $conditions = implode(' OR ', $conditions);

            $this->getSelect()->joinLeft(
                ['cpo' => $this->getTable('catalog_product_option')],
                "cpo.product_id = $mainTableAlias.$linkField",
                ['option_id']
            )->where($conditions);
        } catch (LocalizedException $e) {
            $this->_logger->error($e);
        }

        return $this;
    }

    /**
     * Exclude products with required links
     *
     * @return $this
     */
    public function excludeProductsWithRequiredLinks()
    {
        $this->addAttributeToSelect('links_purchased_separately', 'left');
        $conditions = [];
        $conditions[] = 'at_links_purchased_separately.value != 1';
        $conditions[] = sprintf("%s.type_id != '%s'", self::MAIN_TABLE_ALIAS, DownloadableProduct::TYPE_DOWNLOADABLE);
        $conditions = implode(' OR ', $conditions);
        $this->getSelect()->where($conditions);

        return $this;
    }
}
