<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items\Scenario;

use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items\ScenarioInterface;
use Aheadworks\Afptc\Api\Data\RuleInterface;
use Aheadworks\Afptc\Model\Source\Rule\SimpleAction;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\Scenario\BuyXGetY\Validator as BuyXGetYValidator;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\Scenario\Action\EveryActionConditionFinder;
use Magento\Quote\Model\Quote\Item\AbstractItem;

/**
 * Class BuyXGetY
 *
 * @package Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items\Scenario
 */
class BuyXGetY implements ScenarioInterface
{
    /**
     * @var BuyXGetYValidator
     */
    private $validator;

    /**
     * @var EveryActionConditionFinder
     */
    private $everyActionConditionFinder;

    /**
     * @param BuyXGetYValidator $validator
     * @param EveryActionConditionFinder $everyActionConditionFinder
     */
    public function __construct(
        BuyXGetYValidator $validator,
        EveryActionConditionFinder $everyActionConditionFinder
    ) {
        $this->validator = $validator;
        $this->everyActionConditionFinder = $everyActionConditionFinder;
    }

    /**
     * Retrieve main items of rule
     *
     * @param RuleInterface $rule
     * @param AbstractItem[] $items
     * @return array
     */
    public function getRuleItems($rule, $items)
    {
        $mainItems = [];
        foreach ($items as $item) {
            if (!$this->validator->isValidItem($item, $rule)) {
                continue;
            }

            if (in_array($rule->getSimpleAction(), [SimpleAction::EVERY, SimpleAction::FOR_EACH_N])) {
                if ($this->everyActionConditionFinder->find($rule)) {
                    $mainItems[] = $item;
                    break;
                } else {
                    $mainItems[] = $item;
                }
            } elseif ($rule->getSimpleAction() == SimpleAction::ONLY_ONE_OF) {
                $mainItems[] = $item;
                break;
            }
        }

        return $mainItems;
    }
}
