<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items\Scenario;

use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items\ScenarioInterface;
use Aheadworks\Afptc\Api\Data\RuleInterface;
use Aheadworks\Afptc\Api\Data\ConditionInterface;
use Magento\Quote\Model\Quote\Item\AbstractItem;

/**
 * Class Coupon
 *
 * @package Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items\Scenario
 */
class Coupon implements ScenarioInterface
{
    /**
     * @var BuyXGetY
     */
    private $buyXGetYScenario;

    /**
     * @var array
     */
    private $disabledConditionsType;

    /**
     * @param BuyXGetY $buyXGetYScenario
     * @param array $disabledConditionsType
     */
    public function __construct(
        BuyXGetY $buyXGetYScenario,
        $disabledConditionsType = []
    ) {
        $this->buyXGetYScenario = $buyXGetYScenario;
        $this->disabledConditionsType = $disabledConditionsType;
    }

    /**
     * Retrieve main items of rule
     *
     * @param RuleInterface $rule
     * @param AbstractItem[] $items
     * @return array
     */
    public function getRuleItems($rule, $items)
    {
        $mainItems = [];

        if ($this->getIssetAddressCondition($rule)) {
            return $mainItems;
        }

        return $this->buyXGetYScenario->getRuleItems($rule, $items);
    }

    /**
     * Check whether address condition exists
     *
     * @param RuleInterface $rule
     * @return bool
     */
    private function getIssetAddressCondition($rule)
    {
        if (!$cartConditions = $rule->getCartConditions()) {
            return false;
        }

        return $this->findAddressCondition($cartConditions);
    }

    /**
     * Recursive check
     *
     * @param ConditionInterface $conditions
     * @return bool
     */
    private function findAddressCondition($conditions)
    {
        $result = false;

        if (!$conditions->getConditions()) {
            return $result;
        }

        foreach ($conditions->getConditions() as $cond) {
            if (in_array($cond->getType(), $this->disabledConditionsType)) {
                return true;
            }

            if ($this->findAddressCondition($cond)) {
                return true;
            }
        }

        return $result;
    }
}
