<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items;

use Magento\Framework\Exception\LocalizedException;

/**
 * Class ScenarioPool
 *
 * @package Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Items
 */
class ScenarioPool
{
    /**
     * @var array
     */
    private $scenarios = [];

    /**
     * @param array $scenarios
     */
    public function __construct(
        $scenarios = []
    ) {
        $this->scenarios = $scenarios;
    }

    /**
     * Get scenario processor instance
     *
     * @param string $scenario
     * @return ScenarioInterface
     * @throws LocalizedException
     */
    public function getScenarioProcessor($scenario)
    {
        if (!isset($this->scenarios[$scenario])) {
            throw new LocalizedException(__('Unknown scenario: %s requested', $scenario));
        }
        $scenarioInstance = $this->scenarios[$scenario];
        if (!$scenarioInstance instanceof ScenarioInterface) {
            throw new LocalizedException(
                __('Scenario instance %s does not implement required interface.', $scenario)
            );
        }

        return $scenarioInstance;
    }
}
