<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\Scenario\Action;

use Aheadworks\Afptc\Api\Data\RuleInterface;
use Aheadworks\Afptc\Model\Rule\Converter\Condition as ConditionConverter;
use Aheadworks\Afptc\Model\Rule\Condition\Finder as ConditionFinder;

/**
 * Class EveryActionConditionFinder
 *
 * @package Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\Scenario\Action
 */
class EveryActionConditionFinder
{
    /**
     * @var ConditionConverter
     */
    private $conditionConverter;

    /**
     * @var ConditionFinder
     */
    private $conditionFinder;

    /**
     * @var array
     */
    private $conditions;

    /**
     * @param ConditionConverter $conditionConverter
     * @param ConditionFinder $conditionFinder
     * @param array $conditions
     */
    public function __construct(
        ConditionConverter $conditionConverter,
        ConditionFinder $conditionFinder,
        array $conditions
    ) {
        $this->conditionConverter = $conditionConverter;
        $this->conditionFinder = $conditionFinder;
        $this->conditions = $conditions;
    }

    /**
     * Find specific conditions in rule
     *
     * @param RuleInterface $rule
     * @return bool
     */
    public function find($rule)
    {
        $result = false;
        $conditionArray = $this->conditionConverter->dataModelToArray($rule->getCartConditions());
        foreach ($this->conditions as $index => $params) {
            if ($this->conditionFinder->findInArray($conditionArray, $params)) {
                $result = true;
                break;
            }
        }

        return $result;
    }
}
