<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\Scenario;

use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\ScenarioInterface;
use Aheadworks\Afptc\Model\Source\Rule\SimpleAction;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\Scenario\BuyXGetY\Validator as BuyXGetYValidator;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\Scenario\Action\EveryActionConditionFinder;

/**
 * Class BuyXGetY
 *
 * @package Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\Scenario
 */
class BuyXGetY implements ScenarioInterface
{
    /**
     * @var BuyXGetYValidator
     */
    private $validator;

    /**
     * @var EveryActionConditionFinder
     */
    private $everyActionConditionFinder;

    /**
     * @param BuyXGetYValidator $validator
     * @param EveryActionConditionFinder $everyActionConditionFinder
     */
    public function __construct(
        BuyXGetYValidator $validator,
        EveryActionConditionFinder $everyActionConditionFinder
    ) {
        $this->validator = $validator;
        $this->everyActionConditionFinder = $everyActionConditionFinder;
    }

    /**
     * {@inheritdoc}
     */
    public function getQtyToDiscountByRule($rule, $items)
    {
        $qty = 0;
        foreach ($items as $item) {
            if (!$this->validator->isValidItem($item, $rule)) {
                continue;
            }

            if (in_array($rule->getSimpleAction(), [SimpleAction::EVERY, SimpleAction::FOR_EACH_N])) {
                if ($this->everyActionConditionFinder->find($rule)) {
                    $qty = 1;
                    break;
                } else {
                    $qty += $item->getTotalQty();
                }
            } elseif ($rule->getSimpleAction() == SimpleAction::ONLY_ONE_OF) {
                $qty = 1;
                break;
            }
        }
        if ($rule->getSimpleAction() == SimpleAction::FOR_EACH_N) {
            $qty = floor($qty / $rule->getSimpleActionN());
        }

        return $qty;
    }
}
