<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\StockProduct;

use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\StockProductInterface;
use Magento\CatalogInventory\Api\StockRegistryInterface;
use Magento\CatalogInventory\Model\Stock;
use Magento\Framework\Exception\NoSuchEntityException;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\ProductSourceInterface;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\ProductSourceFactory;

/**
 * Class DefaultStock
 *
 * @package Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\StockProduct
 */
class DefaultStock implements StockProductInterface
{
    /**
     * @var StockRegistryInterface
     */
    private $stockRegistry;

    /**
     * @var ProductSourceInterface
     */
    private $productSource;

    /**
     * DefaultStock constructor.
     *
     * @param StockRegistryInterface $stockRegistry
     * @param ProductSourceFactory   $productSourceFactory
     */
    public function __construct(
        StockRegistryInterface $stockRegistry,
        ProductSourceFactory $productSourceFactory
    ) {
        $this->stockRegistry = $stockRegistry;
        $this->productSource = $productSourceFactory->create();
    }

    /**
     * {@inheritdoc}
     */
    public function getAvailableQty($promoProduct, $scopeId)
    {
        return $this->productSource->getAvailableQty($promoProduct->getProductSku(), $scopeId);
    }

    /**
     * {@inheritdoc}
     */
    public function isManageStockEnabled($promoProduct, $scopeId) : bool
    {
        try {
            $stockItem = $this->stockRegistry->getStockItemBySku($promoProduct->getProductSku(), $scopeId);
            $result = $stockItem->getManageStock();
        } catch (NoSuchEntityException $e) {
            $result = false;
        }

        return $result;
    }

    /**
     * {@inheritdoc}
     */
    public function isBackOrderAvailable($promoProduct, $scopeId) : bool
    {
        try {
            $stockItem = $this->stockRegistry->getStockItemBySku($promoProduct->getProductSku(), $scopeId);
            $result = ($stockItem->getIsInStock() == Stock::STOCK_IN_STOCK
                    && $stockItem->getBackorders() != Stock::BACKORDERS_NO)
                && $stockItem->getManageStock();
        } catch (NoSuchEntityException $e) {
            $result = false;
        }

        return $result;
    }

    /**
     * {@inheritdoc}
     */
    public function getAvailableQtyForIndex($promoProduct, $scopeId)
    {
        return $this->getAvailableQty($promoProduct, $scopeId);
    }
}
