<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty;

use Magento\Framework\Exception\LocalizedException;

/**
 * Class StockProductPool
 *
 * @package Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty
 */
class StockProductPool
{
    /**
     * Default product stock
     */
    const DEFAULT_STOCK = 'default';

    /**
     * @var array
     */
    private $stockProducts = [];

    /**
     * @param array $stockProducts
     */
    public function __construct(
        $stockProducts = []
    ) {
        $this->stockProducts = $stockProducts;
    }

    /**
     * Get stock product instance
     *
     * @param string $productType
     * @return StockProductInterface
     * @throws LocalizedException
     */
    public function getStockProduct($productType)
    {
        if (!isset($this->stockProducts[$productType])) {
            $productType = self::DEFAULT_STOCK;
        }
        $stockProductInstance = $this->stockProducts[$productType];
        if (!$stockProductInstance instanceof StockProductInterface) {
            throw new LocalizedException(
                __('Stock product instance %s does not implement required interface.', $productType)
            );
        }

        return $stockProductInstance;
    }
}
