<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Discount\Calculator;

use Aheadworks\Afptc\Api\Data\RuleMetadataInterface;
use Magento\Quote\Model\Quote\Item\AbstractItem;
use Aheadworks\Afptc\Model\Metadata\Rule\Discount as MetadataRuleDiscount;

/**
 * Interface ItemsCalculatorInterface
 *
 * @package Aheadworks\Afptc\Model\Rule\Discount\Calculator
 */
interface ItemsCalculatorInterface
{
    /**
     * Calculate items discount
     *
     * @param AbstractItem[] $items
     * @param RuleMetadataInterface $metadataRule
     * @param MetadataRuleDiscount $metadataRuleDiscount
     * @return $this
     */
    public function calculate($items, $metadataRule, $metadataRuleDiscount);

    /**
     * Calculate price
     *
     * @param int|float $price
     * @param RuleMetadataInterface $metadataRule
     * @return int|float
     */
    public function calculatePrice($price, $metadataRule);

    /**
     * Calculate discount
     *
     * @param int|float $price
     * @param RuleMetadataInterface $metadataRule
     * @return int|float
     */
    public function calculateDiscount($price, $metadataRule);
}
