<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule;

use Aheadworks\Afptc\Model\Rule as RuleModel;

/**
 * Class Processor
 *
 * @package Aheadworks\Afptc\Model\Rule
 */
class Processor
{
    /**
     * @var array[]
     */
    private $processors;

    /**
     * @param array $processors
     */
    public function __construct(array $processors = [])
    {
        $this->processors = $processors;
    }

    /**
     * Prepare entity data before save
     *
     * @param RuleModel $rule
     * @return RuleModel
     */
    public function prepareDataBeforeSave($rule)
    {
        foreach ($this->processors as $processor) {
            $processor->beforeSave($rule);
        }
        return $rule;
    }

    /**
     * Prepare entity data after load
     *
     * @param RuleModel $rule
     * @return RuleModel
     */
    public function prepareDataAfterLoad($rule)
    {
        foreach ($this->processors as $processor) {
            $processor->afterLoad($rule);
        }
        return $rule;
    }
}
