<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\Promo\Block\Layout\Processor;

use Magento\Framework\Stdlib\ArrayManager;
use Aheadworks\Afptc\Model\Rule\Promo\Block\Layout\Processor\Url\Resolver as UrlResolver;

/**
 * Class Url
 *
 * @package Aheadworks\Afptc\Model\Rule\Promo\Block\Layout\Processor
 */
class Url implements LayoutProcessorInterface
{
    /**
     * @var ArrayManager
     */
    private $arrayManager;

    /**
     * @var UrlResolver
     */
    private $urlResolver;

    /**
     * @param ArrayManager $arrayManager
     * @param UrlResolver $urlResolver
     */
    public function __construct(
        ArrayManager $arrayManager,
        UrlResolver $urlResolver
    ) {
        $this->arrayManager = $arrayManager;
        $this->urlResolver = $urlResolver;
    }

    /**
     * {@inheritdoc}
     */
    public function process($jsLayout, $promoInfoBlock, $placement, $scope)
    {
        $promo = $promoInfoBlock->getPromo();
        $component = 'components/' . $scope;
        $jsLayout = $this->arrayManager->merge(
            $component,
            $jsLayout,
            [
                'popupConfig' => [
                    'url' => $this->urlResolver->resolve($promo->getPromoUrl()),
                    'urlText' => $promo->getPromoUrlText()
                ],
            ]
        );

        return $jsLayout;
    }
}
