<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\PromoOffer\Quote\Address\Processor;

use Magento\Quote\Model\Quote\Address;
use Aheadworks\Afptc\Model\Rule\PromoOffer\Quote\Address\Processor\CartAttribute\Mapper;

/**
 * Class CartAttribute
 *
 * @package Aheadworks\Afptc\Model\Rule\PromoOffer\Quote\Address\Processor
 */
class CartAttribute implements ProcessorInterface
{
    /**
     * @var array
     */
    private $attributes;

    /**
     * @var Mapper
     */
    private $attributeMapper;

    /**
     * @param Mapper $attributeMapper
     * @param array $attributes
     */
    public function __construct(
        Mapper $attributeMapper,
        array $attributes = []
    ) {
        $this->attributeMapper = $attributeMapper;
        $this->attributes = $attributes;
    }

    /**
     * Exclude cart attributes for promo products
     *
     * @param Address $address
     * @param array $data
     * @return array
     */
    public function process($address, $data)
    {
        foreach ($this->attributes as $addressAttribute => $itemAttribute) {
            $attributeValue = $address->getData($addressAttribute);

            foreach ($address->getAllVisibleItems() as $item) {
                if ($item->getAwAfptcIsPromo()) {
                    $attributeValue -= $item->getData($this->attributeMapper->mapAttribute($itemAttribute, $item));
                }
            }
            $data[$addressAttribute] = $attributeValue;
        }

        return $data;
    }
}
