<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Model\Rule\PromoOffer\Validator;

use Aheadworks\Afptc\Model\Rule\Condition\Loader as ConditionLoader;

/**
 * Class AbstractValidator
 *
 * @package Aheadworks\Afptc\Model\Rule\PromoOffer\Validator
 */
class AbstractValidator implements ValidatorInterface
{
    /**
     * @var ConditionLoader
     */
    private $conditionLoader;

    /**
     * @param ConditionLoader $conditionLoader
     */
    public function __construct(
        ConditionLoader $conditionLoader
    ) {
        $this->conditionLoader = $conditionLoader;
    }

    /**
     * {@inheritdoc}
     */
    public function isValidRule($rule, $address, $quoteItem = null)
    {
        $ruleConditions = $this->conditionLoader->loadCartCondition($rule);
        $isValid = $quoteItem
            ? $ruleConditions->validateByEntityId($quoteItem->getProductId())
            : $ruleConditions->validate($address);

        return $isValid;
    }

    /**
     * {@inheritdoc}
     */
    public function isValidItems($items)
    {
        $isValid = false;
        foreach ($items as $item) {
            if (!$item->getAwAfptcIsPromo() && !$item->getParentItem()) {
                $isValid = true;
                break;
            }
        }
        return $isValid;
    }
}
