<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Block\Cart;

use Aheadworks\Afptc\Model\Cart\Item\Renderer\PriceAdjustment;
use Magento\Checkout\Block\Cart\Grid;
use Magento\Msrp\Model\Config;
use Magento\Msrp\Model\Quote\Msrp;

/**
 * Class GridPlugin
 *
 * @package Aheadworks\Afptc\Plugin\Block\Cart
 */
class GridPlugin
{
    /**
     * @var PriceAdjustment
     */
    private $priceAdjustment;

    /**
     * @var Msrp
     */
    private $msrp;

    /**
     * @var Config
     */
    private $msrpConfig;

    /**
     * @param PriceAdjustment $priceAdjustment
     * @param Msrp $msrp
     * @param Config $msrpConfig
     */
    public function __construct(
        PriceAdjustment $priceAdjustment,
        Msrp $msrp,
        Config $msrpConfig
    ) {
        $this->priceAdjustment = $priceAdjustment;
        $this->msrp = $msrp;
        $this->msrpConfig = $msrpConfig;
    }

    /**
     * Get quote items for correct price rendering for promo products
     *
     * @param Grid $block
     */
    public function beforeToHtml(
        Grid $block
    ) {
        $quote = $block->getQuote();
        if (!$this->msrp->getCanApplyMsrp($quote->getId()) && $this->msrpConfig->isEnabled()) {
            foreach ($quote->getItems() as $item) {
                $this->priceAdjustment->adjustForRendering($item);
            }
        }
    }
}
