<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\Converter;

use Aheadworks\Afptc\Model\Formatter\DateTime;
use Magento\SalesRule\Model\Data\Rule as RuleDataModel;
use Magento\SalesRule\Model\Converter\ToModel;
use Magento\SalesRule\Model\Rule;
use Psr\Log\LoggerInterface;

/**
 * Class ToModelPlugin
 * @package Aheadworks\Afptc\Plugin\Model\Converter
 */
class ToModelPlugin
{
    /**
     * @var DateTime
     */
    private $dateTime;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param DateTime $dateTime
     * @param LoggerInterface $logger
     */
    public function __construct(
        DateTime $dateTime,
        LoggerInterface $logger
    ) {
        $this->dateTime = $dateTime;
        $this->logger = $logger;
    }

    /**
     * @param ToModel $subject
     * @param ToModel|Rule $result
     * @param RuleDataModel $dataModel
     * @return $this|Rule
     */
    public function afterToModel(
        ToModel $subject,
        $result,
        RuleDataModel $dataModel
    ) {
        try {
            $result->setFromDate(
                $this->dateTime->formattingDate($dataModel->getFromDate())
            );
            $result->setToDate(
                $this->dateTime->formattingDate($dataModel->getToDate())
            );
        } catch (\Exception $exception) {
            $this->logger->error($exception);
        }

        return $result;
    }
}
