<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\Quote\Item;

use Aheadworks\Afptc\Api\Data\CartItemInterface;
use Magento\Framework\DataObject\Factory as DataObjectFactory;
use Magento\Quote\Model\Quote\Item\CartItemOptionsProcessor;

/**
 * Class CartItemOptionsProcessorPlugin
 * @package Aheadworks\Afptc\Plugin\Model\Quote\Item
 */
class CartItemOptionsProcessorPlugin
{
    /**
     * @var DataObjectFactory
     */
    private $objectFactory;

    /**
     * @param DataObjectFactory $objectFactory
     */
    public function __construct(DataObjectFactory $objectFactory)
    {
        $this->objectFactory = $objectFactory;
    }

    /**
     * Modify request if needed
     *
     * @param CartItemOptionsProcessor $subject
     * @param \Closure $proceed
     * @param $productType
     * @param CartItemInterface $cartItem
     * @return \Magento\Framework\DataObject|float
     */
    public function aroundGetBuyRequest($subject, \Closure $proceed, $productType, $cartItem)
    {
        $buyRequest = $proceed($productType, $cartItem);
        $cartItemExtAttr = $cartItem->getExtensionAttributes();

        if (!$cartItem->getItemId()
            && $cartItemExtAttr && $cartItemExtAttr->getAwAfptcIsPromo()
        ) {
            if (is_numeric($buyRequest)) {
                $buyRequest = $this->objectFactory->create(['qty' => $buyRequest]);
            }
            $buyRequest
                ->setAwAfptcIsPromo($cartItemExtAttr->getAwAfptcIsPromo())
                ->setAwAfptcRules($cartItemExtAttr->getAwAfptcRulesRequest())
                ->setAwAfptcUniqueId($cartItemExtAttr->getAwUniqueId());
        }

        return $buyRequest;
    }
}
