<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\Service;

use Aheadworks\Afptc\Api\CartManagementInterface;
use Aheadworks\Afptc\Api\Data\CartInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Quote\Model\QuoteRepository;
use Psr\Log\LoggerInterface;

/**
 * Class BackendQuoteRepositoryPlugin
 *
 * @package Aheadworks\Afptc\Plugin\Model\Service
 */
class BackendQuoteRepositoryPlugin
{
    /**
     * @var CartManagementInterface
     */
    private $cartManagement;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param CartManagementInterface $cartManagement
     * @param LoggerInterface $logger
     */
    public function __construct(
        CartManagementInterface $cartManagement,
        LoggerInterface $logger
    ) {
        $this->cartManagement = $cartManagement;
        $this->logger = $logger;
    }

    /**
     * Remove unused promo products from cart
     *
     * @param QuoteRepository $subject
     * @param \Closure $proceed
     * @param CartInterface $quote
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @return void
     */
    public function aroundSave($subject, \Closure $proceed, $quote)
    {
        $proceed($quote);
        if ($quote) {
            try {
                $this->cartManagement->removeUnusedPromoData($quote);
            } catch (LocalizedException $e) {
                $this->logger->error($e);
            }
        }
    }
}
