<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\Service;

use Aheadworks\Afptc\Model\ExtensionAttributes\Builder\Creditmemo as CreditmemoExtensionAttributesBuilder;
use Magento\Sales\Api\Data\CreditmemoInterface;
use Magento\Sales\Api\Data\CreditmemoSearchResultInterface;
use Magento\Sales\Api\CreditmemoRepositoryInterface;

/**
 * Class CreditmemoRepositoryPlugin
 * @package Aheadworks\Afptc\Plugin\Model\Service
 */
class CreditmemoRepositoryPlugin
{
    /**
     * @var CreditmemoExtensionAttributesBuilder
     */
    private $creditmemoExtensionAttributesBuilder;

    /**
     * @param CreditmemoExtensionAttributesBuilder $creditmemoExtensionAttributesBuilder
     */
    public function __construct(
        CreditmemoExtensionAttributesBuilder $creditmemoExtensionAttributesBuilder
    ) {
        $this->creditmemoExtensionAttributesBuilder = $creditmemoExtensionAttributesBuilder;
    }

    /**
     * Add data to creditmemo item object
     *
     * @param CreditmemoRepositoryInterface $subject
     * @param CreditmemoSearchResultInterface $result
     * @return CreditmemoSearchResultInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetList($subject, $result)
    {
        foreach ($result->getItems() as $item) {
            $this->creditmemoExtensionAttributesBuilder->attachAwAfptcAttributes($item);
        }
        return $result;
    }

    /**
     * Add data to creditmemo object
     *
     * @param CreditmemoRepositoryInterface $subject
     * @param CreditmemoInterface $item
     * @return CreditmemoInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGet($subject, $item)
    {
        $this->creditmemoExtensionAttributesBuilder->attachAwAfptcAttributes($item);
        return $item;
    }
}
