<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\Service;

use Aheadworks\Afptc\Model\ExtensionAttributes\Builder\Invoice as InvoiceExtensionAttributesBuilder;
use Magento\Sales\Api\Data\InvoiceInterface;
use Magento\Sales\Api\InvoiceRepositoryInterface;
use Magento\Sales\Api\Data\InvoiceSearchResultInterface;

/**
 * Class InvoiceRepositoryPlugin
 * @package Aheadworks\Afptc\Plugin\Model\Service
 */
class InvoiceRepositoryPlugin
{
    /**
     * @var InvoiceExtensionAttributesBuilder
     */
    private $invoiceExtensionAttributesBuilder;

    /**
     * @param InvoiceExtensionAttributesBuilder $invoiceExtensionAttributesBuilder
     */
    public function __construct(
        InvoiceExtensionAttributesBuilder $invoiceExtensionAttributesBuilder
    ) {
        $this->invoiceExtensionAttributesBuilder = $invoiceExtensionAttributesBuilder;
    }

    /**
     * Add data to invoice object
     *
     * @param InvoiceRepositoryInterface $subject
     * @param InvoiceSearchResultInterface $result
     * @return InvoiceSearchResultInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetList($subject, $result)
    {
        foreach ($result->getItems() as $invoice) {
            $this->invoiceExtensionAttributesBuilder->attachAwAfptcAttributes($invoice);
        }
        return $result;
    }

    /**
     * Add data to invoice object
     *
     * @param InvoiceRepositoryInterface $subject
     * @param InvoiceInterface $invoice
     * @return InvoiceInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGet($subject, $invoice)
    {
        $this->invoiceExtensionAttributesBuilder->attachAwAfptcAttributes($invoice);
        return $invoice;
    }
}
