<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\Service;

use Aheadworks\Afptc\Model\ExtensionAttributes\Builder\Order as OrderExtensionAttributesBuilder;
use Magento\Sales\Api\Data\OrderItemInterface;
use Magento\Sales\Api\Data\OrderItemSearchResultInterface;
use Magento\Sales\Api\OrderItemRepositoryInterface;

/**
 * Class OrderItemRepositoryPlugin
 * @package Aheadworks\Afptc\Plugin\Model\Service
 */
class OrderItemRepositoryPlugin
{
    /**
     * @var OrderExtensionAttributesBuilder
     */
    private $orderExtensionAttributesBuilder;

    /**
     * @param OrderExtensionAttributesBuilder $orderExtensionAttributesBuilder
     */
    public function __construct(
        OrderExtensionAttributesBuilder $orderExtensionAttributesBuilder
    ) {
        $this->orderExtensionAttributesBuilder = $orderExtensionAttributesBuilder;
    }

    /**
     * Add data to order item object
     *
     * @param OrderItemRepositoryInterface $subject
     * @param OrderItemSearchResultInterface $result
     * @return OrderItemSearchResultInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetList($subject, $result)
    {
        foreach ($result->getItems() as $item) {
            $this->orderExtensionAttributesBuilder->attachAwAfptcAttributesToItem($item);
        }
        return $result;
    }

    /**
     * Add data to order object
     *
     * @param OrderItemRepositoryInterface $subject
     * @param OrderItemInterface $item
     * @return OrderItemInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGet($subject, $item)
    {
        $this->orderExtensionAttributesBuilder->attachAwAfptcAttributesToItem($item);
        return $item;
    }
}
