<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\Service;

use Aheadworks\Afptc\Model\ExtensionAttributes\Builder\Order as OrderExtensionAttributesBuilder;
use Magento\Sales\Api\Data\OrderInterface;
use Magento\Sales\Api\OrderRepositoryInterface;
use Magento\Sales\Api\Data\OrderSearchResultInterface;

/**
 * Class OrderRepositoryPlugin
 * @package Aheadworks\Afptc\Plugin\Model\Service
 */
class OrderRepositoryPlugin
{
    /**
     * @var OrderExtensionAttributesBuilder
     */
    private $orderExtensionAttributesBuilder;

    /**
     * @param OrderExtensionAttributesBuilder $orderExtensionAttributesBuilder
     */
    public function __construct(
        OrderExtensionAttributesBuilder $orderExtensionAttributesBuilder
    ) {
        $this->orderExtensionAttributesBuilder = $orderExtensionAttributesBuilder;
    }

    /**
     * Add data to order object
     *
     * @param OrderRepositoryInterface $subject
     * @param OrderSearchResultInterface $result
     * @return OrderSearchResultInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetList($subject, $result)
    {
        foreach ($result->getItems() as $order) {
            $this->orderExtensionAttributesBuilder->attachAwAfptcAttributes($order);
        }
        return $result;
    }

    /**
     * Add data to order object
     *
     * @param OrderRepositoryInterface $subject
     * @param OrderInterface $order
     * @return OrderInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGet($subject, $order)
    {
        $this->orderExtensionAttributesBuilder->attachAwAfptcAttributes($order);
        return $order;
    }
}
