<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\Service;

use Aheadworks\Afptc\Model\ExtensionAttributes\Builder\Cart as CartExtensionAttributesBuilder;
use Aheadworks\Afptc\Api\CartManagementInterface;
use Magento\Quote\Api\Data\CartInterface;
use Magento\Framework\Exception\LocalizedException;
use Magento\Quote\Api\CartRepositoryInterface;
use Magento\Quote\Api\Data\CartSearchResultsInterface;
use Magento\Quote\Model\QuoteRepository;
use Psr\Log\LoggerInterface;

/**
 * Class QuoteRepositoryPlugin
 *
 * @package Aheadworks\Afptc\Plugin\Model\Service
 */
class QuoteRepositoryPlugin
{
    /**
     * @var CartExtensionAttributesBuilder
     */
    private $cartExtensionAttributesBuilder;

    /**
     * @var CartManagementInterface
     */
    private $cartManagement;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @param CartExtensionAttributesBuilder $cartExtensionAttributesBuilder
     * @param CartManagementInterface $cartManagement
     * @param LoggerInterface $logger
     */
    public function __construct(
        CartExtensionAttributesBuilder $cartExtensionAttributesBuilder,
        CartManagementInterface $cartManagement,
        LoggerInterface $logger
    ) {
        $this->cartExtensionAttributesBuilder = $cartExtensionAttributesBuilder;
        $this->cartManagement = $cartManagement;
        $this->logger = $logger;
    }

    /**
     * Add data to quote object
     *
     * @param CartRepositoryInterface $subject
     * @param CartSearchResultsInterface $result
     * @return CartSearchResultsInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGetList($subject, $result)
    {
        foreach ($result->getItems() as $cart) {
            $this->cartExtensionAttributesBuilder->attachAwAfptcAttributes($cart);
        }
        return $result;
    }

    /**
     * Add data to quote object
     *
     * @param CartRepositoryInterface $subject
     * @param CartInterface $cart
     * @return CartInterface
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     */
    public function afterGet($subject, $cart)
    {
        $this->cartExtensionAttributesBuilder->attachAwAfptcAttributes($cart);
        return $cart;
    }

    /**
     * Remove unused promo products from cart
     *
     * @param QuoteRepository $subject
     * @param \Closure $proceed
     * @param CartInterface $quote
     * @SuppressWarnings(PHPMD.UnusedFormalParameter)
     * @return void
     */
    public function aroundSave($subject, \Closure $proceed, $quote)
    {
        $proceed($quote);
        $quoteId = $quote->getId();
        if ($quoteId) {
            try {
                $this->cartManagement->removeUnusedPromoData($quoteId);
            } catch (LocalizedException $e) {
                $this->logger->error($e);
            }
        }
    }
}
