<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Plugin\Model\ThirdPartyModule\RewardPoints\Calculator;

use Aheadworks\RewardPoints\Model\Calculator\Spending;
use Magento\Quote\Api\Data\CartItemInterface;
use Magento\Quote\Model\Quote\Item\AbstractItem;

/**
 * Class SpendingPlugin
 *
 * @package Aheadworks\Afptc\Plugin\Model\ThirdPartyModule\RewardPoints\Calculator
 */
class SpendingPlugin
{
    /**
     * Retrieve item total
     *
     * @param Spending $subject
     * @param float|int $result
     * @param CartItemInterface $item
     * @param float $itemBasePrice
     * @return float|int
     */
    public function afterGetItemTotal(
        Spending $subject,
        $result,
        CartItemInterface $item,
        $itemBasePrice
    ) {
        if ($item->getAwAfptcIsPromo()) {
            $percent = (100 - $item->getAwAfptcPercent()) / 100;
            $result = $result * $percent;
        }

        return $result;
    }

    /**
     * Retrieve item price with discount
     *
     * @param Spending $subject
     * @param float|int $result
     * @param AbstractItem $item
     * @param float|int $itemPrice
     * @return float|int
     */
    public function afterGetItemPriceWithDiscount(
        Spending $subject,
        $result,
        AbstractItem $item,
        $itemPrice
    ) {
        // Calculate item price with discount for bundle dynamic product
        if ($item->getAwAfptcIsPromo()) {
            $availablePercent = (100 - $item->getAwAfptcPercent()) / 100;
            $result = $itemPrice * $availablePercent * $item->getTotalQty();
        }

        return $result;
    }

    /**
     * Retrieve base item price with discount
     *
     * @param Spending $subject
     * @param float|int $result
     * @param AbstractItem $item
     * @param float|int $baseItemPrice
     * @return float|int
     */
    public function afterGetItemBasePriceWithDiscount(
        Spending $subject,
        $result,
        AbstractItem $item,
        $baseItemPrice
    ) {
        // Calculate item price with discount for bundle dynamic product
        if ($item->getAwAfptcIsPromo()) {
            $availablePercent = (100 - $item->getAwAfptcPercent()) / 100;
            $result = $baseItemPrice * $availablePercent * $item->getTotalQty();
        }

        return $result;
    }
}
