<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Test\Unit\Model\Rule\Converter\ToMetadataRule\Resolver\Qty;

use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use PHPUnit\Framework\TestCase;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\ScenarioPool;
use Aheadworks\Afptc\Model\Rule\Converter\ToMetadataRule\Resolver\Qty\ScenarioInterface;

/**
 * Class ScenarioPoolTest
 *
 * @package Aheadworks\Afptc\Test\Unit\Model\Rule\Converter\ToMetadataRule\Resolver\Qty
 */
class ScenarioPoolTest extends TestCase
{
    /**
     * @var ScenarioPool
     */
    private $model;

    /**
     * @var ScenarioInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    private $scenarioMock;

    /**
     * Init mocks for tests
     *
     * @return void
     */
    public function setUp(): void
    {
        $objectManager = new ObjectManager($this);
        $this->scenarioMock = $this->getMockForAbstractClass(ScenarioInterface::class);
        $this->model = $objectManager->getObject(
            ScenarioPool::class,
            [
                'scenarios' => [
                    'coupon' => $this->scenarioMock
                ]
            ]
        );
    }

    /**
     * Test for getStockProduct method
     */
    public function testGetScenarioProcessor()
    {
        $type = 'coupon';
        $this->assertSame($this->scenarioMock, $this->model->getScenarioProcessor($type));
    }

    /**
     * Test for getScenarioProcessor method with non-existing type
     *
     * @expectedException \Exception
     */
    public function testGetStockProductWithNonExistingType()
    {
        $type = 'non-existing-type';
        $this->expectException(LocalizedException::class);
        $this->model->getScenarioProcessor($type);
    }
}
