<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace  Aheadworks\Afptc\Test\Unit\Model\Rule\Discount\Calculator\Item;

use Aheadworks\Afptc\Model\Metadata\Rule\Discount\Item as MetadataRuleDiscountItem;
use Aheadworks\Afptc\Model\Metadata\Rule\Discount\ItemFactory as MetadataRuleDiscountItemFactory;
use Magento\Framework\Pricing\PriceCurrencyInterface;
use Magento\Framework\Api\SimpleDataObjectConverter;
use Aheadworks\Afptc\Model\Rule\Discount\Calculator\Item\Distributor;
use Aheadworks\Afptc\Model\Rule\Discount\Calculator\Item\Processor;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use PHPUnit\Framework\TestCase;
use Magento\Quote\Model\Quote\Item\AbstractItem;

/**
 * Class DistributorTest
 *
 * @package Aheadworks\Afptc\Test\Unit\Model\Rule\Discount\Calculator\Item
 */
class DistributorTest extends TestCase
{
    /**
     * @var Distributor
     */
    private $model;

    /**
     * @var MetadataRuleDiscountItemFactory|\PHPUnit_Framework_MockObject_MockObject
     */
    private $metadataRuleDiscountItemFactoryMock;

    /**
     * @var SimpleDataObjectConverter|\PHPUnit_Framework_MockObject_MockObject
     */
    private $simpleDataObjectConverterMock;

    /**
     * @var PriceCurrencyInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    private $priceCurrencyMock;

    /**
     * @var Processor|\PHPUnit_Framework_MockObject_MockObject
     */
    private $processorMock;

    /**
     * Init mocks for tests
     *
     * @return void
     */
    public function setUp(): void
    {
        $objectManager = new ObjectManager($this);
        $this->metadataRuleDiscountItemFactoryMock = $this->createMock(MetadataRuleDiscountItemFactory::class);
        $this->simpleDataObjectConverterMock = $this->createMock(SimpleDataObjectConverter::class);
        $this->priceCurrencyMock = $this->getMockForAbstractClass(PriceCurrencyInterface::class);
        $this->processorMock = $this->createMock(Processor::class);
        $this->model = $objectManager->getObject(
            Distributor::class,
            [
                'metadataRuleDiscountItemFactory' => $this->metadataRuleDiscountItemFactoryMock,
                'simpleDataObjectConverter' => $this->simpleDataObjectConverterMock,
                'priceCurrency' => $this->priceCurrencyMock,
                'processor' => $this->processorMock
            ]
        );
    }

    /**
     * Test for distribute method
     */
    public function testDistribute()
    {
        $metadataRuleDiscountItemMock = $this->createMock(MetadataRuleDiscountItem::class);
        $itemMock = $this->getMockBuilder(AbstractItem::class)
            ->disableOriginalConstructor()
            ->setMethods([
                'getAddress',
                'getQuote',
                'getOptionByCode',
                'getHasChildren',
                'isChildrenCalculated',
                'getChildren'
            ])->getMock();

        $metadataRuleDiscountItemMock->expects($this->exactly(2))
            ->method('getItem')
            ->willReturn($itemMock);
        $itemMock->expects($this->once())
            ->method('getHasChildren')
            ->willReturn(true);
        $itemMock->expects($this->once())
            ->method('isChildrenCalculated')
            ->willReturn(true);
        $qtyToDiscount = 5;
        $metadataRuleDiscountItemMock->expects($this->once())
            ->method('getQty')
            ->willReturn($qtyToDiscount);
        $parentBaseTotal = 10;
        $this->processorMock->expects($this->once())
            ->method('getTotalItemBasePrice')
            ->with($itemMock, $qtyToDiscount)
            ->willReturn($parentBaseTotal);

        $itemMock->expects($this->once())
            ->method('getChildren')
            ->willReturn([]);

        $metadataRuleDiscountItemMock->expects($this->once())
            ->method('setAmount')
            ->with(0)
            ->willReturnSelf();
        $metadataRuleDiscountItemMock->expects($this->once())
            ->method('setBaseAmount')
            ->with(0)
            ->willReturnSelf();
        $metadataRuleDiscountItemMock->expects($this->once())
            ->method('setChildren')
            ->with([])
            ->willReturnSelf();

        $this->assertEquals(
            $metadataRuleDiscountItemMock,
            $this->model->distribute($metadataRuleDiscountItemMock)
        );
    }
}
