<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace  Aheadworks\Afptc\Test\Unit\Model\Rule\Discount\Calculator\Item;

use Magento\Quote\Model\Quote\Item\AbstractItem;
use Aheadworks\Afptc\Model\Rule\Discount\Calculator\Item\Processor;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use PHPUnit\Framework\TestCase;

/**
 * Class ProcessorTest
 *
 * @package Aheadworks\Afptc\Model\Rule\Discount\Calculator\Item
 */
class ProcessorTest extends TestCase
{
    /**
     * @var Processor
     */
    private $model;

    /**
     * Init mocks for tests
     *
     * @return void
     */
    public function setUp(): void
    {
        $objectManager = new ObjectManager($this);
        $this->model = $objectManager->getObject(Processor::class, []);
    }

    /**
     * Test for getTotalItemPrice method
     *
     * @dataProvider getTotalItemPriceDataProvider
     * @param int|null $qty
     * @param bool $hasChildren
     * @param bool $isChildrenCalculated
     * @param int|float $result
     */
    public function testGetTotalItemPrice($qty, $hasChildren, $isChildrenCalculated, $result)
    {
        /** @var AbstractItem|\PHPUnit_Framework_MockObject_MockObject $itemMock */
        $itemMock = $this->getMockBuilder(AbstractItem::class)
            ->disableOriginalConstructor()
            ->setMethods([
                'getAddress',
                'getQuote',
                'getOptionByCode',
                'getTotalQty',
                'getHasChildren',
                'isChildrenCalculated',
                'getChildren',
                'getAwRafAmount',
                'getDiscountAmount',
                'getDiscountCalculationPrice',
                'getCalculationPrice'
            ])->getMock();

        if (!$qty) {
            $itemMock->expects($this->once())
                ->method('getTotalQty')
                ->willReturn(5);
        }

        $itemMock->expects($this->once())
            ->method('getHasChildren')
            ->willReturn($hasChildren);
        $itemMock->expects($this->any())
            ->method('isChildrenCalculated')
            ->willReturn($isChildrenCalculated);

        $itemMock->expects($this->once())
            ->method('getDiscountCalculationPrice')
            ->willReturn(30);
        $itemMock->expects($this->once())
            ->method('getCalculationPrice')
            ->willReturn(25);

        if ($hasChildren && $isChildrenCalculated) {
            $childItem = clone $itemMock;
            $itemMock->expects($this->once())
                ->method('getChildren')
                ->willReturn([$childItem]);

            $childItem->expects($this->once())
                ->method('getDiscountAmount')
                ->willReturn(8);
        } else {
            $itemMock->expects($this->once())
                ->method('getDiscountAmount')
                ->willReturn(7);
        }

        $itemMock->expects($this->once())
            ->method('getAwRafAmount')
            ->willReturn(4);

        $this->assertSame($result, $this->model->getTotalItemPrice($itemMock, $qty));
    }

    /**
     * Test for getBaseTotalItemPrice method
     *
     * @dataProvider getTotalItemPriceDataProvider
     * @param int|null $qty
     * @param bool $hasChildren
     * @param bool $isChildrenCalculated
     * @param int|float $result
     */
    public function testGetBaseTotalItemPrice($qty, $hasChildren, $isChildrenCalculated, $result)
    {
        /** @var AbstractItem|\PHPUnit_Framework_MockObject_MockObject $itemMock */
        $itemMock = $this->getMockBuilder(AbstractItem::class)
            ->disableOriginalConstructor()
            ->setMethods([
                'getAddress',
                'getQuote',
                'getOptionByCode',
                'getTotalQty',
                'getHasChildren',
                'isChildrenCalculated',
                'getChildren',
                'getBaseAwRafAmount',
                'getBaseDiscountAmount',
                'getDiscountCalculationPrice',
                'getBaseDiscountCalculationPrice'
            ])->getMock();

        if (!$qty) {
            $itemMock->expects($this->once())
                ->method('getTotalQty')
                ->willReturn(5);
        }

        $itemMock->expects($this->once())
            ->method('getHasChildren')
            ->willReturn($hasChildren);
        $itemMock->expects($this->any())
            ->method('isChildrenCalculated')
            ->willReturn($isChildrenCalculated);

        $itemMock->expects($this->once())
            ->method('getDiscountCalculationPrice')
            ->willReturn(30);
        $itemMock->expects($this->once())
            ->method('getBaseDiscountCalculationPrice')
            ->willReturn(30);

        if ($hasChildren && $isChildrenCalculated) {
            $childItem = clone $itemMock;
            $itemMock->expects($this->once())
                ->method('getChildren')
                ->willReturn([$childItem]);

            $childItem->expects($this->once())
                ->method('getBaseDiscountAmount')
                ->willReturn(8);
        } else {
            $itemMock->expects($this->once())
                ->method('getBaseDiscountAmount')
                ->willReturn(7);
        }

        $itemMock->expects($this->once())
            ->method('getBaseAwRafAmount')
            ->willReturn(4);

        $this->assertSame($result, $this->model->getTotalItemBasePrice($itemMock, $qty));
    }

    /**
     * Data provider for testGetTotalItemPrice method
     */
    public function getTotalItemPriceDataProvider()
    {
        return [
            [3, true, true, 78],
            [null, true, true, 138],
            [3, false, true, 79],
            [3, true, false, 79],
            [3, false, false, 79]
        ];
    }
}
