<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Test\Unit\Model\Rule\Discount\Calculator;

use InvalidArgumentException;
use Aheadworks\Afptc\Model\Rule\Discount\Calculator\Pool;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use PHPUnit\Framework\TestCase;
use Aheadworks\Afptc\Model\Rule\Discount\Calculator\DiscountCalculatorInterface;
use Aheadworks\Afptc\Model\Source\Rule\Discount\Type as DiscountType;

/**
 * Class PoolTest
 *
 * @package Aheadworks\Afptc\Test\Unit\Model\Rule\Discount\Calculator
 */
class PoolTest extends TestCase
{
    /**
     * @var Pool
     */
    private $model;

    /**
     * @var DiscountCalculatorInterface|\PHPUnit_Framework_MockObject_MockObject
     */
    private $byPercentCaclulatorMock;

    /**
     * Init mocks for tests
     *
     * @return void
     */
    public function setUp(): void
    {
        $objectManager = new ObjectManager($this);
        $this->byPercentCaclulatorMock = $this->getMockForAbstractClass(DiscountCalculatorInterface::class);
        $this->model = $objectManager->getObject(
            Pool::class,
            [
                'calculators' => [
                    'percent' => $this->byPercentCaclulatorMock
                ]
            ]
        );
    }

    /**
     * Test for getCalculatorByType method
     */
    public function testGetCalculatorByType()
    {
        $type = DiscountType::PERCENT;
        $this->assertSame($this->byPercentCaclulatorMock, $this->model->getCalculatorByType($type));
    }

    /**
     * Test for getCalculatorByType method on exception
     *
     * @expectedException \InvalidArgumentException
     */
    public function testGetCalculatorByTypeOnException()
    {
        $type = 'non-existing-type';
        $this->expectException(InvalidArgumentException::class);
        $this->model->getCalculatorByType($type);
    }
}
