<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Processor;

use Aheadworks\Afptc\Api\Data\PromoOfferRender\ProductRenderInterface;
use Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Processor\QuantityNotice\Resolver as NoticeResolver;

/**
 * Class QuantityNotice
 *
 * @package Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Processor
 */
class QuantityNotice implements ProcessorInterface
{
    /**
     * @var NoticeResolver
     */
    private $noticeResolver;

    /**
     * @param NoticeResolver $noticeResolver
     */
    public function __construct(
        NoticeResolver $noticeResolver
    ) {
        $this->noticeResolver = $noticeResolver;
    }

    /**
     * {@inheritdoc}
     */
    public function prepareRender($promoOfferRender, $metadataRules)
    {
        $items = $promoOfferRender->getItems();
        $promoOfferRender->setIsQuantityNoticeActive($this->checkIfNoticeIsVisible($items));
    }

    /**
     * Check if notice is visible
     *
     * @param ProductRenderInterface[] $items
     * @return bool
     */
    private function checkIfNoticeIsVisible($items)
    {
        foreach ($items as $item) {
            if ($this->noticeResolver->isNeedToDisplayForItem($item)) {
                return true;
            }
        }
        return false;
    }
}
