<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Processor\QuantityNotice;

use Magento\Framework\Serialize\Serializer\Json;
use Aheadworks\Afptc\Api\Data\PromoOfferRender\ProductRenderInterface;
use Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Product\Processor\Option\ConfigurationPool;
use Psr\Log\LoggerInterface;
use Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Product\Processor\Option\Type as OptionType;

/**
 * Class Resolver
 *
 * @package Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Processor\QuantityNotice
 */
class Resolver
{
    /**
     * @var ConfigurationPool
     */
    private $configurationPool;

    /**
     * @var LoggerInterface
     */
    private $logger;

    /**
     * @var Json
     */
    private $serializer;

    /**
     * @param ConfigurationPool $configurationPool
     * @param LoggerInterface $logger
     * @param Json $serializer
     */
    public function __construct(
        ConfigurationPool $configurationPool,
        LoggerInterface $logger,
        Json $serializer
    ) {
        $this->configurationPool = $configurationPool;
        $this->logger = $logger;
        $this->serializer = $serializer;
    }

    /**
     * Resolve if notice should be displayed
     *
     * @param ProductRenderInterface $item
     * @return bool
     */
    public function isNeedToDisplayForItem($item)
    {
        $productType = $item->getType();
        $result = false;
        if ($this->configurationPool->hasConfiguration($productType)) {
            try {
                $configuration = $this->configurationPool->getConfiguration($productType);
                $option = $this->serializer->unserialize($item->getOption());
                $optionType = $configuration->getOptionType($option);
                $result = $optionType == OptionType::CONFIGURABLE_OPTION;
            } catch (\Exception $e) {
                $this->logger->error($e);
            }
        }
        return $result;
    }
}
