<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Product\Processor\Option;

use Magento\Framework\Exception\LocalizedException;

/**
 * Class ConfigurationPool
 *
 * @package Aheadworks\Afptc\Ui\DataProvider\PromoOffer\RenderList\Product\Processor\Configuration
 */
class ConfigurationPool
{
    /**
     * @var array
     */
    private $configurations = [];

    /**
     * @param array $configurations
     */
    public function __construct(
        $configurations = []
    ) {
        $this->configurations = $configurations;
    }

    /**
     * Get configuration instance
     *
     * @param string $productType
     * @return ConfigurationInterface
     * @throws LocalizedException
     */
    public function getConfiguration($productType)
    {
        if (!isset($this->configurations[$productType])) {
            throw new LocalizedException(__('Unknown configuration: %s requested', $productType));
        }
        $configurationInstance = $this->configurations[$productType];
        if (!$configurationInstance instanceof ConfigurationInterface) {
            throw new LocalizedException(
                __('Configuration instance %s does not implement required interface.', $productType)
            );
        }

        return $configurationInstance;
    }

    /**
     * Check if configuration for product type exists
     *
     * @param string $productType
     * @return bool
     */
    public function hasConfiguration($productType)
    {
        return isset($this->configurations[$productType]);
    }
}
