<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Ui\DataProvider\Rule\FormDataProcessor;

use Aheadworks\Afptc\Model\Rule\Image\Manager as ImageManager;
use Aheadworks\Afptc\Api\Data\RuleInterface;

/**
 * Class Image
 *
 * @package Aheadworks\Afptc\Ui\DataProvider\Rule\FormDataProcessor
 */
class Image implements ProcessorInterface
{
    /**
     * @var ImageManager
     */
    private $imageManager;

    /**
     * @param ImageManager $imageManager
     */
    public function __construct(
        ImageManager $imageManager
    ) {
        $this->imageManager = $imageManager;
    }

    /**
     * {@inheritdoc}
     */
    public function prepareEntityData($data)
    {
        if (isset($data[RuleInterface::PROMO_IMAGE]) && $imageFileName = $data[RuleInterface::PROMO_IMAGE]) {
            try {
                $url = $this->imageManager->getUrl($imageFileName);
            } catch (\Exception $exception) {
                $url = null;
            }
            if ($url) {
                $alt = isset($data[RuleInterface::PROMO_IMAGE_ALT_TEXT])
                    ? $data[RuleInterface::PROMO_IMAGE_ALT_TEXT]
                    : '';
                $data[RuleInterface::PROMO_IMAGE] = [
                    0 => [
                        'file' => $imageFileName,
                        'url'  => $url,
                        'type' => 'image',
                        'alt'  => $alt
                    ]
                ];
            }
        }
        return $data;
    }

    /**
     * {@inheritdoc}
     */
    public function prepareMetaData($meta)
    {
        return $meta;
    }
}
