<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    Afptc
 * @version    1.5.4
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\Afptc\Ui\DataProvider\Rule\Product;

use Magento\Catalog\Ui\DataProvider\Product\ProductDataProvider;
use Magento\Catalog\Model\ResourceModel\Product\CollectionFactory as MagentoCollectionFactory;
use Aheadworks\Afptc\Model\ResourceModel\Product\Collection;
use Aheadworks\Afptc\Model\ResourceModel\Product\CollectionFactory;
use Magento\Catalog\Api\Data\ProductInterface;
use Magento\Catalog\Model\Product\Visibility as ProductVisibility;
use Aheadworks\Afptc\Model\Source\Product\AllowedType;
use Magento\Framework\Model\ResourceModel\Db\Collection\AbstractCollection;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class ListingDataProvider
 *
 * @package Aheadworks\Afptc\Ui\DataProvider\Rule\Product
 */
class ListingDataProvider extends ProductDataProvider
{
    /**
     * @var ProductVisibility
     */
    private $productVisibility;

    /**
     * @var AllowedType
     */
    private $allowedProductType;

    /**
     * @var StoreManagerInterface
     */
    private $storeManager;

    public function __construct(
        $name,
        $primaryFieldName,
        $requestFieldName,
        MagentoCollectionFactory $magentoCollectionFactory,
        ProductVisibility $productVisibility,
        AllowedType $allowedProductType,
        StoreManagerInterface $storeManager,
        CollectionFactory $productCollectionFactory,
        array $addFieldStrategies = [],
        array $addFilterStrategies = [],
        array $meta = [],
        array $data = []
    ) {
        parent::__construct(
            $name,
            $primaryFieldName,
            $requestFieldName,
            $magentoCollectionFactory,
            $addFieldStrategies,
            $addFilterStrategies,
            $meta,
            $data
        );
        $this->productVisibility = $productVisibility;
        $this->allowedProductType = $allowedProductType;
        $this->storeManager = $storeManager;
        $this->collection = $productCollectionFactory->create();
    }

    /**
     * Get data
     *
     * @return array
     */
    public function getData()
    {
        $collection = $this->getCollection();
        $collection->setStoreId($this->storeManager->getStore()->getId());
        $this->applyLimitations($collection);
        return parent::getData();
    }

    /**
     * Apply limitations to product collection
     *
     * @param Collection|AbstractCollection $collection
     */
    public function applyLimitations($collection)
    {
        //todo it would be good to decompose this approach to make collection more flexible
        $collection->addAttributeToSelect(ProductInterface::STATUS);
        $collection->addAttributeToFilter(
            ProductInterface::TYPE_ID,
            ['in' => $this->allowedProductType->getTypeList()]
        );
        $collection->excludeProductsWithRequiredOptions();
        $collection->excludeProductsWithRequiredLinks();
    }
}
