<?php
/**
 * Aheadworks Inc.
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the EULA
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * https://ecommerce.aheadworks.com/end-user-license-agreement/
 *
 * @package    AfptcKlarna
 * @version    1.0.0
 * @copyright  Copyright (c) 2020 Aheadworks Inc. (http://www.aheadworks.com)
 * @license    https://ecommerce.aheadworks.com/end-user-license-agreement/
 */
namespace Aheadworks\AfptcKlarna\Test\Unit\Model\Checkout\Orderline;

use Aheadworks\AfptcKlarna\Model\Checkout\Orderline\Afptc;
use Klarna\Core\Helper\DataConverter;
use Magento\Framework\App\Config\ScopeConfigInterface;
use PHPUnit\Framework\TestCase;
use Magento\Framework\TestFramework\Unit\Helper\ObjectManager;
use Klarna\Core\Api\BuilderInterface;
use Magento\Quote\Model\Quote;
use Magento\Quote\Model\Quote\Address\Total;

/**
 * Class AfptcTest
 * @package Aheadworks\AfptcKlarna\Test\Unit\Model\Checkout\Orderline
 */
class AfptcTest extends TestCase
{
    /**
     * @var Afptc
     */
    private $model;

    /**
     * @var BuilderInterface
     */
    private $checkoutMock;

    /**
     * Init mocks for tests
     *
     * @return void
     */
    public function setUp(): void
    {
        $objectManager = new ObjectManager($this);
        $this->checkoutMock = $this->getMockForAbstractClass(
            BuilderInterface::class,
            [], '', true, true, true, [
                'addData',
                'getAwAfptcReference',
                'getAwAfptcTotalAmount',
                'addOrderLine',
                'getAwAfptcTitle',
                'getAwAfptcUnitPrice',
                'getAwAfptcTaxRate',
                'getAwAfptcTaxAmount',
                'getObject'
            ]
        );
        $this->model = $objectManager->getObject(Afptc::class);
    }

    /**
     * Test collect method
     */
    public function testCollect()
    {
        $quoteMock = $this->createPartialMock(Quote::class, ['getTotals']);
        $totalMock = $this->getMockBuilder(Total::class)
            ->disableOriginalConstructor()
            ->setMethods(['getValue', 'getTitle', 'getCode'])
            ->getMock();
        $helperMock = $this->createMock(DataConverter::class);
        $title = 'title';
        $code = 'code';
        $value = 1;

        $this->checkoutMock->expects($this->once())
            ->method('getObject')
            ->willReturn($quoteMock);
        $quoteMock->expects($this->once())
            ->method('getTotals')
            ->willReturn(['aw_afptc' => $totalMock]);
        $totalMock->expects($this->once())
            ->method('getValue')
            ->willReturn($value);
        $helperMock->expects($this->any())
            ->method('toApiFloat')
            ->with($value)
            ->willReturn($value);
        $totalMock->expects($this->once())
            ->method('getTitle')
            ->willReturn($title);
        $totalMock->expects($this->once())
            ->method('getCode')
            ->willReturn($code);

        $this->checkoutMock->expects($this->once())
            ->method('addData')
            ->with([
                'aw_afptc_unit_price' => null,
                'aw_afptc_tax_rate' => 0,
                'aw_afptc_total_amount' => null,
                'aw_afptc_tax_amount' => 0,
                'aw_afptc_title' => $title,
                'aw_afptc_reference' => $code
            ])
            ->willReturn($this->checkoutMock);

        $this->assertEquals($this->model, $this->model->collect($this->checkoutMock));
    }

    /**
     * Test fetch method
     */
    public function testFetch()
    {
        $reference = 1;
        $title = 'title';
        $unitPrice = 1;
        $taxRate = 1;
        $totalAmount = 1;
        $taxAmount = 1;

        $this->checkoutMock->expects($this->any())
            ->method('getAwAfptcReference')
            ->willReturn($reference);
        $this->checkoutMock->expects($this->any())
            ->method('getAwAfptcTotalAmount')
            ->willReturn($totalAmount);
        $this->checkoutMock->expects($this->once())
            ->method('getAwAfptcTitle')
            ->willReturn($title);
        $this->checkoutMock->expects($this->once())
            ->method('getAwAfptcUnitPrice')
            ->willReturn($unitPrice);
        $this->checkoutMock->expects($this->once())
            ->method('getAwAfptcTaxRate')
            ->willReturn($taxRate);
        $this->checkoutMock->expects($this->once())
            ->method('getAwAfptcTaxAmount')
            ->willReturn($taxAmount);
        $this->checkoutMock->expects($this->once())
            ->method('addOrderLine')
            ->with([
                'type' => Afptc::ITEM_TYPE_AFPTC,
                'reference' => $reference,
                'name' => $title,
                'quantity' => 1,
                'unit_price' => $unitPrice,
                'tax_rate' => $taxRate,
                'total_amount' => $totalAmount,
                'total_tax_amount' => $taxAmount
            ])->willReturn($this->checkoutMock);

        $this->assertEquals($this->model, $this->model->fetch($this->checkoutMock));
    }
}
